/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief  XMEGA SPI interrupt-driven driver example source.
 *
 *      This file contains an example application that demonstrates the SPI
 *      interrupt-driven driver.
 *
 * \par Application note:
 *      AVR1309: Using the XMEGA SPI
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 763 $
 * $Date: 2007-11-06 14:51:56 +0100 (ti, 06 nov 2007) $  \n
 *
 * Copyright (c) 2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "avr_compiler.h"
#include "spi_driver_2wire.h"

/* Global variables. */


/*! \brief SPI slave on PORT D. */
SPI_Slave_t spiSlaveC;
uint8_t bytesToRecieve=0;
uint8_t bytesToTransmit=0;
uint8_t counter1=0;
uint8_t counter2=0;
uint8_t* dat;
bool transmisionStarted= false;

//uint8_t dat[10];


void osc_init(void){
//++++++++++++++++++++ externer Quarz 16Mhz auf 32Mhz(intern) +++++++++++++++++++++++++++++++++++++++++
	
	OSC.XOSCCTRL|=  OSC_XOSCSEL_XTAL_16KCLK_gc| OSC_FRQRANGE_12TO16_gc ;
	OSC.PLLCTRL= OSC_PLLSRC_XOSC_gc | (1<<OSC_PLLFAC1_bp);  // PLL auf externen Osc einstellen & *2

	OSC.CTRL |= (1<<OSC_PLLEN_bp) | (1<<OSC_XOSCEN_bp);// PLL & externen OSC enable

	while(!(OSC.STATUS & OSC_PLLRDY_bm));

	CCP = CCP_IOREG_gc; //IOProtect

	CLK.CTRL |= CLK_SCLKSEL_PLL_gc; // PLL als Clock auswhlen
}	


void ir_init(void){
	/* Enable  medium level interrupt in the interrupt controller. */
		PMIC.CTRL |= PMIC_MEDLVLEN_bm ;
		sei();

	}

int main( void )
{

	//internen Quarz mit 32 MHZ initialisieren
	osc_init();


	/* Initialize SPI slave on port C */
	SPI_SlaveInit(&spiSlaveC, 
	              &SPIC,
	              &PORTC,
	              false,		
	              SPI_MODE_3_gc,
	              SPI_INTLVL_MED_gc);


	ir_init();

//tue nichts bis ein Interrupt auftritt	
	while(true) {
		nop();	
	}

}



ISR(SPIC_INT_vect)
{	


	/* Empfangene Daten in Buffer schreiben*/
	uint8_t data = SPI_SlaveReadByte(&spiSlaveC);
	
	/* Bei einer neuen bertragung ist BytesToReceive =0)*/
	if(transmisionStarted== false){
		transmisionStarted=true;
		/*obere 4 Bytes des ersten Datums ist die Anzahl der vom Master
		* gesendeten Bytes*/
		bytesToRecieve = (data & 0xf0)>>4;
		/* Die unteren 4 Bytes sind die Anzahl der Bytes die der Master 
		* erwartet. */
		bytesToTransmit= data & 0x0f;
		/* Speicher reservieren je nachdem wie viel Bytes vom Master kommen*/
		dat= (uint8_t*) malloc(bytesToRecieve * sizeof(uint8_t));
		/* Counter auf initial wert*/
		counter1=0;
		counter2=0;
		
	}else if(counter1 < bytesToRecieve){
		dat[counter1]=data;
		counter1++;
		/* wenn alle Daten empfangen*/
		if(counter1==bytesToRecieve){
			if(bytesToTransmit==0){
				transmisionStarted=false;
				return;
				}
			/* MISO als Ausgang schalten*/
			PORTC.DIRSET = SPI_MISO_bm;
			/* invertierte Daten an Master senden*/
			data = ~dat[counter2];
			SPI_SlaveWriteByte(&spiSlaveC, data);
			counter2++;


		}
	}else if(counter2 < bytesToTransmit){
//		PORTC.DIRSET = SPI_MISO_bm;
		data = ~dat[counter2];
		SPI_SlaveWriteByte(&spiSlaveC, data);
		counter2++;	

	}else if(counter2 == bytesToTransmit){
		/* nachdem alle Daten Empfangen und 
		*  gesendet wurden nun wieder Speicher freigeben
		*  und MISO auf Eingang schalten,
		*  sowie bytesToRecieve wieder auf null setzten damit
		*  klar ist das nchste Datum was kommt ist das indem die Anzahl der 
		* zu empfangenden/ sendenden Bytes steht 	*/
		uint8_t *old_dat;
		old_dat= dat;
		free (old_dat);
		PORTC.DIR &= ~SPI_MISO_bm;
		bytesToRecieve=0;
		bytesToTransmit=0;
		transmisionStarted=false;
	}
	


}
